// This file is part of Awali.
// Copyright 2016-2023 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include <awali/sttc/tests/null_stream.hxx>

#include<awali/sttc/ctx/lan_char.hh>
#include<awali/sttc/ctx/lal_char.hh>
#include<awali/sttc/weightset/tupleset.hh>

#include<awali/sttc/automaton.hh>
#include<awali/sttc/weightset/z.hh>
#include<awali/sttc/weightset/zmin.hh>
#include<awali/sttc/weightset/q.hh>
#include<awali/sttc/weightset/r.hh>
#include<awali/sttc/weightset/c.hh>
#include<awali/sttc/weightset/f2.hh>

using namespace awali::sttc;


int main(int argc, char **argv) {
  // Context of 4-tape transducer
  std::ostream * osc;
  if(argc==2)
    osc = &std::cout;
  else
    osc = &null_stream;

  *osc << "Build a transducer" << std::endl;
  
  using context_t = context<ctx::lat<ctx::lan_char,ctx::lal_char>,b>;
  context_t ctx(std::make_tuple(ctx::lan_char({'a','b'}),
			       ctx::lal_char({'x','y'})),
		b());
  auto tdc=make_shared_ptr<mutable_automaton<context_t>>(ctx);

  *osc << "Build a NFA" << std::endl;
  auto autb = make_automaton({'a','b'});
  *osc << "Build a Z-WFA" << std::endl;
  auto autz = make_automaton<z>({'a','b'});
  *osc << "Build a (Z,min,+)-WFA" << std::endl;
  auto autzmin = make_automaton<zmin>({'a','b'});
  *osc << "Build a Q-WFA" << std::endl;
  auto autq = make_automaton<q>({'a','b'});
  *osc << "Build a R-WFA" << std::endl;
  auto autr = make_automaton<r>({'a','b'});
  *osc << "Build a C-WFA" << std::endl;
  auto autc = make_automaton<c>({'a','b'});
  *osc << "Build a F2-WFA" << std::endl;
  auto autf2 = make_automaton<f2>({'a','b'});

  *osc << "Build a NFA with epsilon transitions" << std::endl;
  auto autbe = make_automaton_with_epsilon({'a','b'});
  *osc << "Build a Z-WFA with epsilon transitions" << std::endl;
  auto autze = make_automaton_with_epsilon<z>({'a','b'});
  *osc << "Build a (Z,min,+)-WFA with epsilon transitions" << std::endl;
  auto autzmine = make_automaton_with_epsilon<zmin>({'a','b'});
  *osc << "Build a Q-WFA with epsilon transitions" << std::endl;
  auto autqe = make_automaton_with_epsilon<q>({'a','b'});
  *osc << "Build a R-WFA with epsilon transitions" << std::endl;
  auto autre = make_automaton_with_epsilon<r>({'a','b'});
  *osc << "Build a C-WFA with epsilon transitions" << std::endl;
  auto autce = make_automaton_with_epsilon<c>({'a','b'});
  *osc << "Build a F2-WFA with epsilon transitions" << std::endl;
  auto autf2e = make_automaton_with_epsilon<f2>({'a','b'});
  return 0;
}
